----------------------------------------------------------------------------------
--  File: top.vhd
----------------------------------------------------------------------------------
--  Author: Jordan Retz
--  CopyRight (c) 2013 Digilent Inc.
----------------------------------------------------------------------------------
--  Revision History:
--  	07/25/2013	Created (Jordan Retz)
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VComponents.all;


----------------------------------------------------------------------------------
--	Title: top Entity
----------------------------------------------------------------------------------
--  Inputs: 
--  Outputs: 
----------------------------------------------------------------------------------
--  Description:
--  	
----------------------------------------------------------------------------------
entity top is
	Port ( 
			-- FPGA_GCLK, 8MHz
			CLK			: in STD_LOGIC;
			
			-- FPGA_LFC, 1 Hz
			CLK_LFC		: in STD_LOGIC;
			
			-- LED outputs
			LED_0			: out STD_LOGIC;
			LED_1			: out STD_LOGIC;
			LED_2			: out STD_LOGIC;
			LED_3			: out STD_LOGIC;
			
			-- BTN inputs
			BTN_0			: in STD_LOGIC;
			BTN_1			: in STD_LOGIC;
			
			-- DEPP address strobe (ASTB) and DEPP data strobe (DSTB)
         DEPP_ASTB 	: in  STD_LOGIC;
         DEPP_DSTB 	: in  STD_LOGIC;
			  
			-- DEPP write enable (write operation = 0, read operation = 1)
         DEPP_WRITE 	: in  STD_LOGIC;
			  
			-- DEPP logic ready 
			DEPP_WAIT	: out STD_LOGIC;
			  
			-- DEPP bi-directional data bus. Direction controlled by DEPP_WRITE
         DBUS 			: inout  STD_LOGIC_VECTOR (7 downto 0);
			
			-- connected to physical I/O pad
			PORTA 		: inout  STD_LOGIC_VECTOR (7 downto 0);
			PORTB 		: inout  STD_LOGIC_VECTOR (7 downto 0);
			PORTC 		: inout  STD_LOGIC_VECTOR (6 downto 0);
			PORTD 		: inout  STD_LOGIC_VECTOR (7 downto 0);
			PORTE 		: inout  STD_LOGIC_VECTOR (7 downto 0);	
			PORTF 		: inout  STD_LOGIC_VECTOR (6 downto 0)	
		);
end top;

-------------------------------------------------------
-------------------------------------------------------
-- Architecture Start
-------------------------------------------------------
-------------------------------------------------------

architecture Behavioral of top is

-------------------------------------------------------
-- Sub module interconnect signals
-------------------------------------------------------
signal port_a_top 	: STD_LOGIC_VECTOR (7 downto 0);
signal port_b_top 	: STD_LOGIC_VECTOR (7 downto 0);
signal port_c_top 	: STD_LOGIC_VECTOR (6 downto 0);
signal port_d_top 	: STD_LOGIC_VECTOR (7 downto 0);
signal port_e_top 	: STD_LOGIC_VECTOR (7 downto 0);
signal port_f_top 	: STD_LOGIC_VECTOR (6 downto 0);

signal bus_cntrl_to_io : STD_LOGIC_VECTOR (7 downto 0);
signal bus_io_to_cntrl : STD_LOGIC_VECTOR (7 downto 0);

signal bus_addr		: STD_LOGIC_VECTOR (3 downto 0);

signal hs : STD_LOGIC;

-------------------------------------------------------
-- 50 MHz clock
-------------------------------------------------------
signal clk_50 : STD_LOGIC;

-------------------------------------------------------
-- Component Declaration
-------------------------------------------------------
component port_io
	Port 	( CLK : in STD_LOGIC;
			  ADDRESS : in  STD_LOGIC_VECTOR (3 downto 0);
			  RW	: in STD_LOGIC;
			  HS	: in STD_LOGIC;
			  REG_OUT : out STD_LOGIC_VECTOR (7 downto 0);
			  REG_IN	 : in STD_LOGIC_VECTOR (7 downto 0);
			  PORTA : inout  STD_LOGIC_VECTOR (7 downto 0);
			  PORTB : inout  STD_LOGIC_VECTOR (7 downto 0);
			  PORTC : inout  STD_LOGIC_VECTOR (6 downto 0);
			  PORTD : inout  STD_LOGIC_VECTOR (7 downto 0);
			  PORTE : inout  STD_LOGIC_VECTOR (7 downto 0);	
			  PORTF : inout  STD_LOGIC_VECTOR (6 downto 0)
		 );
end component;

component depp_cntrl
	Port	( CLK 				: in  STD_LOGIC;
			  DEPP_ASTB 		: in  STD_LOGIC;
			  DEPP_DSTB 		: in  STD_LOGIC;
			  DEPP_WRITE 		: in  STD_LOGIC;
			  DEPP_WAIT			: out STD_LOGIC;
			  DBUS 				: inout  STD_LOGIC_VECTOR (7 downto 0);
			  INTERNAL_DOUT 	: out  STD_LOGIC_VECTOR (7 downto 0);
			  INTERNAL_DIN 	: in  STD_LOGIC_VECTOR (7 downto 0);
			  INTERNAL_AOUT 	: out  STD_LOGIC_VECTOR (3 downto 0);
			  INTERNAL_HS		: out STD_LOGIC
			);
end component;

component clk_gen_50MHz is
port	( CLK_IN1           : in     std_logic;
		  CLK_OUT1          : out    std_logic
		);
end component;

-------------------------------------------------------
-- Behavioral Begin
-------------------------------------------------------
begin

-------------------------------------------------------
-- Component Instantiation
-------------------------------------------------------
CLK_GEN: clk_gen_50MHz port map	( CLK_IN1 => CLK,
											  CLK_OUT1 => clk_50);

PORTIO: port_io port map	( CLK => clk_50,
									  ADDRESS => bus_addr,
									  RW => DEPP_WRITE,
									  HS => hs,
									  REG_OUT => bus_io_to_cntrl,
									  REG_IN => bus_cntrl_to_io,
							 		  PORTA => PORTA,
							 		  PORTB => PORTB,
									  PORTC => PORTC,
									  PORTD => PORTD,
									  PORTE => PORTE,
									  PORTF => PORTF
									);
DEPPCNTRL: depp_cntrl port map	(  CLK => clk_50,
												DEPP_ASTB => DEPP_ASTB,
												DEPP_DSTB => DEPP_DSTB,
												DEPP_WRITE => DEPP_WRITE,
												DEPP_WAIT => DEPP_WAIT,
												DBUS => DBUS,
												INTERNAL_DOUT => bus_cntrl_to_io,
												INTERNAL_DIN => bus_io_to_cntrl,
												INTERNAL_AOUT => bus_addr,
												INTERNAL_HS => hs
											);

-------------------------------------------------------
-- LED/BTN Testing
-------------------------------------------------------
LED_3 <= CLK_LFC;
LED_2 <= not CLK_LFC;

LED_0 <= BTN_0;
LED_1 <= BTN_1;

end Behavioral;

